proc budg_init_var {org_id proj_id budg_id}\
{
	global BUDG_REC_${org_id}_${proj_id}_${budg_id}
	set BUDG_REC_${org_id}_${proj_id}_${budg_id} ""
}

proc budg_new_id {org_id proj_id}\
{
	global LIST_BUDG
	set budg_id 0
	foreach b $LIST_BUDG \
	{
		if {[regexp "^${org_id}_${proj_id}_(\[0-9\]+)" $b all id]}\
		{
			if {$id > $budg_id}\
			{
				set budg_id $id
			}
		}
	}
	return [expr $budg_id + 1]
}

proc budg_d_new {org_id}\
{
	global proj
	global CUR_ORG LIST_PROJ LIST_BUDG
	catch {destroy .newbudg}
	toplevel .newbudg
	wm title .newbudg "New budget"
	wm geometry .newbudg +50+50
	frame .newbudg.b
	pack .newbudg.b -side bottom -pady 5 -padx 5
		button .newbudg.b.ok -text Ok -width 10 \
			-command {
				if {[.newbudg.l.f3.lb size] == 0}\
				{
					return
				}
				set proj_id [join [lindex [split $proj ---] 0]]
				set budg_id [budg_new_id $CUR_ORG $proj_id]
				budg_init_var $CUR_ORG $proj_id $budg_id
				upvar #0 BUDG_REC_${CUR_ORG}_${proj_id}_${budg_id} BUDG_REC
				set BUDG_REC ""
				upvar #0 BUDG_${CUR_ORG}_${proj_id}_${budg_id} BUDG
				set BUDG ""
				if {[join [.newbudg.name.e get]] == ""}\
				{
					keylset BUDG name "Untitled"
				}\
				else\
				{
					keylset BUDG name [.newbudg.name.e get]
				}
				lappend LIST_BUDG ${CUR_ORG}_${proj_id}_${budg_id}
				budg_save $CUR_ORG $proj_id $budg_id
				o_menu_compose $CUR_ORG
				destroy .newbudg
			}
		pack .newbudg.b.ok -side left
		button .newbudg.b.cancel -text Cancel -width 10 \
			-command "
				destroy .newbudg
			"
		pack .newbudg.b.cancel -side left
	frame .newbudg.name
	pack .newbudg.name -fill x
		label .newbudg.name.l -text "Budget's name: "
		pack .newbudg.name.l -side left
		entry .newbudg.name.e
		pack .newbudg.name.e -side left
	frame .newbudg.l
	pack .newbudg.l -side left -fill y -pady 5 -padx 5
		frame .newbudg.l.f3
		pack .newbudg.l.f3
			scrollbar .newbudg.l.f3.vs -command {.newbudg.l.f3.lb yview}
			pack .newbudg.l.f3.vs -side right -fill y
			listbox .newbudg.l.f3.lb -width 30 -yscrollcommand {.newbudg.l.f3.vs set}
			pack .newbudg.l.f3.lb -side right -fill y
			foreach p $LIST_PROJ \
			{
				global PROJ_${org_id}_$p
				.newbudg.l.f3.lb insert end "$p --- [keylget PROJ_${org_id}_$p name]"
			}
			.newbudg.l.f3.lb selection set 0
			set proj [.newbudg.l.f3.lb get 0]
			bind .newbudg.l.f3.lb <1> {
				update
				set proj [.newbudg.l.f3.lb get @%x,%y]
			}
	focus .newbudg.name.e
}

proc budg_save {org_id proj_id budg_id}\
{
	global ORG_PATH
	upvar #0 BUDG_${org_id}_${proj_id}_${budg_id} BUDG
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	set ct [catch {
		set F [open $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}.tmp w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError $errmsg
		return
	}
	set ct [catch {
	puts $F "set BUDG_${org_id}_${proj_id}_${budg_id} \"\""
	puts $F "\n# STRUCTURE"
	puts $F "keylset BUDG_${org_id}_${proj_id}_${budg_id} name \"[keylget BUDG name]\""
	puts $F "\n# RECORDS"
	puts $F "set BUDG_REC_${org_id}_${proj_id}_${budg_id} \"\""
	foreach i [keylkeys BUDG_REC] \
	{
		puts $F "keylset BUDG_REC_${org_id}_${proj_id}_${budg_id} $i [list [keylget BUDG_REC $i]]"
	}
	close $F
	} errmsg]
	if {$ct > 0}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}.tmp
		myError "Cannot save budg. Reason:\n$errmsg"
	}\
	else\
	{
		frename $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}.tmp $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}
		chmod 0600 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}
	}
}

proc budg_export_ascii {org_id proj_id budg_id}\
{
	upvar #0 BUDG_${org_id}_${proj_id}_${budg_id} BUDG
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	global FILENAME BUDG_SORT
	o_wait
	budg_save $org_id $proj_id $budg_id
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the budget as text. Reason:\n$errmsg"
		return
	}
	puts $F "[keylget BUDG name]\n"
	puts $F "+----------+--------------------------------------------------+--------+"
	set line [format "|%-10s|%-50.50s|%8s|" "Date" "Description" "Amount"]
	puts $F "$line"
	puts $F "+----------+--------------------------------------------------+--------+"
	foreach i [lsort -command "budg_sort $org_id $proj_id $budg_id $BUDG_SORT" [keylkeys BUDG_REC]] \
	{
		set line [format "|%10s|%-50.50s|%8.2f|" [keylget BUDG_REC $i.date] [keylget BUDG_REC $i.description] [keylget BUDG_REC $i.amount]]
		puts $F "$line"
	}
	puts $F "+----------+--------------------------------------------------+--------+"
	set line [format "|%10s|%50.50s|%8.2f|" " " "Total" [budg_total $org_id $proj_id $budg_id]]
	puts $F "$line"
	puts $F "+----------+--------------------------------------------------+--------+"
	close $F
	o_idle
}

proc budg_export_html {org_id proj_id budg_id}\
{
	upvar #0 BUDG_${org_id}_${proj_id}_${budg_id} BUDG
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	global FILENAME BUDG_SORT
	o_wait
	budg_save $org_id $proj_id $budg_id
	if {[file extension $FILENAME] != ".html"}\
	{
		append FILENAME ".html"
	}
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the budget as html. Reason:\n$errmsg"
		return
	}
	puts $F "<html><head>"
	puts $F "<title>[html_text2html [keylget BUDG name]]</title></head>"
	puts $F "<body bgcolor=\"#FFFFFF\">"
	puts $F "<h2>[html_text2html [keylget BUDG name]]</h2>"
	puts $F "<table>"
	puts $F "\t<tr><td><b>Date</b></td><td><b>Description</b></td><td><b>Amount</b></td></tr>"
	foreach i [lsort -command "budg_sort $org_id $proj_id $budg_id $BUDG_SORT" [keylkeys BUDG_REC]] \
	{
		puts $F "\t<tr><td>[keylget BUDG_REC $i.date]</td><td>[html_text2html [keylget BUDG_REC $i.description]]</td><td align=\"right\">[keylget BUDG_REC $i.amount]</td></tr>"
	}
	puts $F "\t<tr><td>&nbsp;</td><td align=\"right\"><b>Total</b></td><td align=\"right\"><b>[format "%.2f" [budg_total $org_id $proj_id $budg_id]]</b></td></tr>"
	puts $F "</table>"
	puts $F "</body></html>"
	close $F
	o_idle
}

proc budg_d_export {org_id proj_id budg_id mode}\
{
	global FILENAME fbe 
	catch {destroy .file}
	toplevel .file
	wm title .file "Export budget - $mode"
	wm geometry .file 250x250+100+50
	filebrowser .file.fb 
	pack .file.fb
		if {$mode == "html"}\
		{
			.file.fb addlistbox .*.html -selectmode single
		}\
		else\
		{
			.file.fb addlistbox .* -selectmode single
		}
		.file.fb addlabel -text "Export: "
		.file.fb adddirlabel
		set fbe [.file.fb addentry -width 20]
		.file.fb addbutton -text OK -width 10 \
			-command "
				set FILENAME \[$fbe get\]
				if {\$FILENAME != \"\"}\
				{
					destroy .file
					budg_export_$mode $org_id $proj_id $budg_id
				}
			"
		.file.fb addbutton -text Cancel -width 10 \
			-command "destroy .file"
		if {$mode == "html"}\
		{
			.file.fb readdir [pwd] .*.html
		}\
		else\
		{
			.file.fb readdir [pwd] .*
		}
	focus $fbe
}

proc budg_delete {org_id proj_id budg_id}\
{
	global ORG_PATH LIST_BUDG CUR_BUDG CUR_TYPE
	global BUDG_${org_id}_${proj_id}_${budg_id}
	global BUDG_REC_${org_id}_${proj_id}_${budg_id}
	global confirm
	gen_d_yn_confirm "Do you really want to delete \"[keylget BUDG_${org_id}_${proj_id}_${budg_id} name]\"?"
	tkwait variable confirm
	if {$confirm == "yes"}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/budg_${org_id}_${proj_id}_${budg_id}
		set LIST_BUDG [lindex [intersect3 $LIST_BUDG ${org_id}_${proj_id}_${budg_id}] 0]
		unset BUDG_${org_id}_${proj_id}_${budg_id}
		unset BUDG_REC_${org_id}_${proj_id}_${budg_id}
		o_menu_compose $org_id
		set CUR_TYPE proj
		o_display_proj $org_id $proj_id
	}
}

##############################

proc budg_total {org_id proj_id budg_id}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	set tot 0.00
	foreach i [keylkeys BUDG_REC]\
	{
		set tot [expr $tot + [keylget BUDG_REC $i.amount]]
	}
	return $tot
}

##############################

proc budg_sort {org_id proj_id budg_id field id1 id2}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	if {$field == "id"}\
	{
		if {$id1 > $id2}\
		{
			return 1
		}\
		elseif {$id1 < $id2}\
		{
			return -1
		}\
		else\
		{
			return 0
		}
	}\
	else\
	{
		return [ccollate [keylget BUDG_REC $id1.$field] [keylget BUDG_REC $id2.$field]]
	}
}

proc budg_d_ask_description {org_id proj_id budg_id id}\
{
	upvar #0 BUDG_${org_id}_${proj_id}_${budg_id} BUDG
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	global description amount date
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Description"
	wm geometry .ask +100+100
	if {[keylget BUDG_REC $id.amount {}]}\
	{
		set amount [keylget BUDG_REC $id.amount]
	}\
	else\
	{
		set amount "0.00"
	}
	if {[keylget BUDG_REC $id.date {}]}\
	{
		set date [keylget BUDG_REC $id.date]
	}\
	else\
	{
		set date [gen_get_cur_year]-[gen_get_cur_month]-[gen_get_cur_day]
	}
	set y [lindex [split $date -] 0]
	set m [lindex [split $date -] 1]
	set d [lindex [split $date -] 2]
	text .ask.description -width 30 -height 1 -wrap word
	pack .ask.description -side top -fill x -anchor w
	frame .ask.date
	pack .ask.date -side top
		label .ask.date.lb -text "Date (YYYY MM DD)" -width 20
		pack .ask.date.lb -side left
		entry .ask.date.year -width 4
		pack .ask.date.year -side left
		entry .ask.date.month -width 2
		pack .ask.date.month -side left
		entry .ask.date.day -width 2
		pack .ask.date.day -side left
	.ask.date.year insert 0 $y
	.ask.date.month insert 0 $m
	.ask.date.day insert 0 $d
	frame .ask.amount
	pack .ask.amount -side top
		label .ask.amount.lb -text "Amount: "
		pack .ask.amount.lb -side left
		entry .ask.amount.e -width 10
		pack .ask.amount.e -side left
	.ask.amount.e insert 0 $amount
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				if {! [regexp {^[0-9]+(\.[0-9]+)?$} [.ask.amount.e get]]}\
				{
					myError "The amount should be a decimal number."
				}\
				else\
				{
					if {[gen_check_date [.ask.date.year get] [.ask.date.month get] [.ask.date.day get] "Date"]}\
					{
						regsub -all "^0*" [.ask.date.year get] "" y
						regsub -all "^0*" [.ask.date.month get] "" m
						regsub -all "^0*" [.ask.date.day get] "" d
						set date [format "%04d-%02d-%02d" $y $m $d]
						set amount [format "%.2f" [.ask.amount.e get]]
						set description "[.ask.description get 1.0 end-1c]"
						destroy .ask
					}
				}
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
		button .ask.b.del -text Delete -width 10 \
			-command "
				set description \"\"
				budg_del_record $org_id $proj_id $budg_id $id
				destroy .ask
			"
		pack .ask.b.del -side left
	if {[keylget BUDG_REC $id.description {}]}\
	{
		.ask.description insert 1.0 "[keylget BUDG_REC $id.description]"
		.ask.b.cancel configure -command "
			set amount $amount
			set date $date
			set description [list [keylget BUDG_REC $id.description]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set description \"\"
			destroy .ask
		"
	}
	focus .ask.description
}

proc budg_popup_description {org_id proj_id budg_id id}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	global description amount date
	budg_d_ask_description $org_id $proj_id $budg_id $id
	tkwait variable description
	if {$description == ""}\
	{
		if {[keylget BUDG_REC $id {}]}\
		{
			keyldel BUDG_REC $id
		}
		return
	}
	keylset BUDG_REC $id.description "$description"
	keylset BUDG_REC $id.amount "$amount"
	keylset BUDG_REC $id.date "$date"
	.r.budg.c itemconfigure date$id -text "[keylget BUDG_REC $id.date]"
	.r.budg.c itemconfigure desc$id -text "[keylget BUDG_REC $id.description]"
	.r.budg.c itemconfigure am$id -text "[keylget BUDG_REC $id.amount]" -anchor ne
	set tot [budg_total $org_id $proj_id $budg_id]
	.r.budg.tot.val configure -text [format "%.2f" $tot]
}

proc budg_new_record {org_id proj_id budg_id}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	upvar #0 TOT_NUM_${org_id}_${proj_id}_${budg_id} TOT_NUM
	global description amount date CUR_Y
	set CUR_ID [expr [lindex [lsort [keylkeys BUDG_REC]] end] + 1]
	budg_d_ask_description $org_id $proj_id $budg_id $CUR_ID
	tkwait variable description
	if {$description == ""}\
	{
		if {[keylget BUDG_REC $CUR_ID {}]}\
		{
			keyldel BUDG_DEL $CUR_ID
		}
		return
	}
	keylset BUDG_REC $CUR_ID.date "$date"
	keylset BUDG_REC $CUR_ID.description "$description"
	keylset BUDG_REC $CUR_ID.amount "$amount"
	set tagrect [.r.budg.c create rectangle 10 [expr $CUR_Y - 5] 420 [expr $CUR_Y + 20] -fill #FFFFFF -outline {} -tags rect$CUR_ID]
	set tag1 [.r.budg.c create text 20 $CUR_Y -text "[keylget BUDG_REC $CUR_ID.date]" -anchor nw -tags date$CUR_ID]
	set tag2 [.r.budg.c create text 100 $CUR_Y -text "[keylget BUDG_REC $CUR_ID.description]" -anchor nw -width 350 -tags desc$CUR_ID]
	set tag3 [.r.budg.c create text 380 $CUR_Y -text "[keylget BUDG_REC $CUR_ID.amount]" -anchor ne -tags am$CUR_ID]
	incr CUR_Y 30
	.r.budg.c bind $tagrect <1> "
		budg_popup_description $org_id $proj_id $budg_id $CUR_ID
	"
	.r.budg.c bind $tag1 <1> "
		budg_popup_description $org_id $proj_id $budg_id $CUR_ID
	"
	.r.budg.c bind $tag2 <1> "
		budg_popup_description $org_id $proj_id $budg_id $CUR_ID
	"
	.r.budg.c bind $tag3 <1> "
		budg_popup_description $org_id $proj_id $budg_id $CUR_ID
	"
	.r.budg.c configure -scrollregion [.r.budg.c bbox all]
	set tot [budg_total $org_id $proj_id $budg_id]
	.r.budg.tot.val configure -text [format "%.2f" $tot]
}

proc budg_del_record {org_id proj_id budg_id id}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	if {! [keylget BUDG_REC $id {}]}\
	{
		return
	}
	keyldel BUDG_REC $id
	budg_create_list $org_id $proj_id $budg_id
}

#############################

proc budg_create_list {org_id proj_id budg_id}\
{
	upvar #0 BUDG_REC_${org_id}_${proj_id}_${budg_id} BUDG_REC
	global BUDG_COL CUR_Y BUDG_SORT
	.r.budg.c delete all
	.r.budg.c create rectangle 0 0 420 40 -fill #EEEEEE -outline {}
	set tag [.r.budg.c create text 20 20 -text "Date" -anchor nw]
	.r.budg.c bind $tag <1> "
		set BUDG_SORT date
		budg_create_list $org_id $proj_id $budg_id
	"
	set tag [.r.budg.c create text 100 20 -text "Description" -anchor nw]
	.r.budg.c bind $tag <1> "
		set BUDG_SORT description
		budg_create_list $org_id $proj_id $budg_id
	"
	set tag [.r.budg.c create text 350 20 -text "Amount" -anchor n]
	.r.budg.c bind $tag <1> "
		set BUDG_SORT amount
		budg_create_list $org_id $proj_id $budg_id
	"
	set CUR_Y 50
	foreach i [lsort -command "budg_sort $org_id $proj_id $budg_id $BUDG_SORT" [keylkeys BUDG_REC]] \
	{
		set tagrect [.r.budg.c create rectangle 10 [expr $CUR_Y - 5] 400 [expr $CUR_Y + 20] -fill #FFFFFF -outline {} -tags rect$i]
		set tag1 [.r.budg.c create text 20 $CUR_Y -text "[keylget BUDG_REC $i.date]" -anchor nw -tags date$i]
		set tag2 [.r.budg.c create text 100 $CUR_Y -text "[keylget BUDG_REC $i.description]" -anchor nw -width 350 -tags desc$i]
		set tag3 [.r.budg.c create text 380 $CUR_Y -text "[keylget BUDG_REC $i.amount]" -anchor ne -tags am$i]
		incr CUR_Y 30
		.r.budg.c bind $tag1 <1> "
			budg_popup_description $org_id $proj_id $budg_id $i
		"
		.r.budg.c bind $tag2 <1> "
			budg_popup_description $org_id $proj_id $budg_id $i
		"
		.r.budg.c bind $tagrect <1> "
			budg_popup_description $org_id $proj_id $budg_id $i
		"
	}
	.r.budg.c configure -scrollregion [.r.budg.c bbox all]
	set tot [budg_total $org_id $proj_id $budg_id]
	.r.budg.tot.val configure -text [format "%.2f" $tot]
}

