proc gen_last_day {year month day}\
{
	regsub "^0" $year "" year
	regsub "^0" $month "" month
	regsub "^0" $day "" day
	if {$day < 28}\
	{
		return 0
	}
	if {$day == 31 && [lsearch "1 3 5 7 8 10 12" $month] >= 0}\
	{
		return 1
	}\
	elseif {$day == 30 && [lsearch "4 6 9 11" $month] >= 0}\
	{
		return 1
	}\
	elseif {$month == 2}\
	{
		if {$day == 28 && [expr $year % 100] == 0 && [expr $year % 400] != 0}\
		{
			return 1
		}\
		elseif {$day == 28 && [expr $year % 4] != 0}\
		{
			return 1
		}\
		elseif {$day == 29}\
		{
			return 1
		}\
		else\
		{
			return 0
		}
	}\
	else\
	{
		return 0
	}
}

proc gen_get_day {year month day}\
{
	regsub "^0" $year "" year
	regsub "^0" $month "" month
	regsub "^0" $day "" day
	if {$day < 29 || [lsearch "1 3 5 7 8 10 12" $month] >= 0}\
	{
		return $day
	}\
	else\
	{
		if {$month != 2 && ($day == 29 || $day == 30)}\
		{
			return $day
		}\
		elseif {$month != 2}\
		{
			return 30
		}\
		elseif {$month == 2}\
		{
			if {[expr $year % 100] == 0 && [expr $year % 400] != 0}\
			{
				return 28
			}\
			elseif {[expr $year % 4] == 0}\
			{
				return 29
			}\
			else\
			{
				return 28
			}
		}
	}
}

proc gen_get_weekday {year month day}\
{
	return [clock format [clock scan $month/$day/$year] -format "%w"]
}

proc gen_get_weekday_name {year month day}\
{
	return [clock format [clock scan $month/$day/$year] -format "%A"]
}

proc gen_get_month_name {month}\
{
	return [clock format [clock scan $month/1/1999] -format "%B"]
}

proc gen_get_cur_year {}\
{
	global CUR_YEAR
	if {$CUR_YEAR == ""}\
	{
		set CUR_YEAR [clock format [clock seconds] -format "%Y"]
	}
	return $CUR_YEAR
}

##############################

proc gen_check_date {year month day {error ""}}\
{
	if {$year == ""}\
	{
		myError "$error\nThe year is empty."
		return 0
	}
	if {$month == ""}\
	{
		myError "$error\nThe month is empty."
		return 0
	}
	if {$day == ""}\
	{
		myError "$error\nThe day is empty."
		return 0
	}
	set year [string trimleft $year 0]
	if {! [ctype digit $year]}\
	{
		myError "$error\nThe year must consist of 4 digits representing a year."
		return 0
	}
	set month [string trimleft $month 0]
	if {! [ctype digit $month] || $month < 1 || $month > 12}\
	{
		myError "$error\nThe month must consist of digits from 1 to 12."
		return 0
	}
	set day [string trimleft $day 0]
	if {! [ctype digit $day] || $day < 1 || $day > 31}\
	{
		myError "$error\nThe day must consist of digits from 1 to 31."
		return 0
	}
	if {[clength $year] != 4}\
	{
		myError "$error\nThe year must be in 4 digit format (YYYY)."
		return 0
	}
	set day2 [gen_get_day $year $month $day]
	if {$day != $day2}\
	{
		myError "$error\nThe month of [gen_get_month_name $month] only have $day2 days."
		return 0
	}
	return 1
}

##############################

proc gen_get_cur_month {}\
{
	global CUR_MONTH
	if {$CUR_MONTH == ""}\
	{
		set CUR_MONTH [clock format [clock seconds] -format "%m"]
	}
	return $CUR_MONTH
}

proc gen_get_cur_day {}\
{
	global CUR_DAY
	if {$CUR_DAY == ""}\
	{
		set CUR_DAY [clock format [clock seconds] -format "%e"]
	}
	return $CUR_DAY
}

proc gen_d_yn_confirm {str}\
{
	catch {destroy .confirm}
	toplevel .confirm
	wm title .confirm "Confirmation"
	wm geometry .confirm +100+100
	label .confirm.lb -text "$str" -wraplength 300 -justify left
	pack .confirm.lb -side top -padx 10 -pady 10
	frame .confirm.b
	pack .confirm.b -side top
		button .confirm.b.yes -text "Yes" -width 10 \
			-command {
				set confirm "yes"
				destroy .confirm
			}
		pack .confirm.b.yes -side left
		button .confirm.b.no -text "No" -width 10 \
			-command {
				set confirm "no"
				destroy .confirm
			}
		pack .confirm.b.no -side left
}
		
