proc proj_init_var {org_id proj_id}\
{
	global PROJ_${org_id}_${proj_id}
	set PROJ_${org_id}_${proj_id} ""
}

proc proj_new_id {org_id}\
{
	global LIST_PROJ
	set proj_id 0
	foreach b $LIST_PROJ \
	{
		if {[regexp "^(\[0-9\]+)" $b all id]}\
		{
			if {$id > $proj_id}\
			{
				set proj_id $id
			}
		}
	}
	return [expr $proj_id + 1]
}

proc proj_d_new {org_id}\
{
	global proj
	global CUR_ORG LIST_PROJ
	catch {destroy .newproj}
	toplevel .newproj
	wm title .newproj "New project"
	wm geometry .newproj +50+50
	frame .newproj.b
	pack .newproj.b -side bottom -pady 5 -padx 5
		button .newproj.b.ok -text Ok -width 10 \
			-command {
				set proj_id [proj_new_id $CUR_ORG]
				proj_init_var $CUR_ORG $proj_id
				upvar #0 PROJ_${CUR_ORG}_${proj_id} PROJ
				set PROJ ""
				if {[join [.newproj.name.e get]] == ""}\
				{
					keylset PROJ name "Untitled"
				}\
				else\
				{
					keylset PROJ name [.newproj.name.e get]
				}
				lappend LIST_PROJ ${proj_id}
				proj_save $CUR_ORG $proj_id
				o_menu_compose $CUR_ORG
				destroy .newproj
			}
		pack .newproj.b.ok -side left
		button .newproj.b.cancel -text Cancel -width 10 \
			-command "
				destroy .newproj
			"
		pack .newproj.b.cancel -side left
	frame .newproj.name
	pack .newproj.name -fill x
		label .newproj.name.l -text "Project's name: "
		pack .newproj.name.l -side left
		entry .newproj.name.e
		pack .newproj.name.e -side left
	focus .newproj.name.e
}

proc proj_save {org_id proj_id}\
{
	upvar #0 PROJ_${org_id}_${proj_id} PROJ
	global ORG_PATH
	set ct [catch {
		set F [open $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc.tmp w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "$errmsg"
		return
	}
	if {! [file exists $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}]}\
	{
		mkdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}
		chmod 0700 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}
	}
	set ct [catch {
		puts $F "# STRUCTURE\n"
		puts $F "set PROJ_${org_id}_${proj_id} \"\"\n"
		puts $F "keylset PROJ_${org_id}_${proj_id} name [list [keylget PROJ name]]\n"
		foreach f [readdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}] \
		{
			if {[regexp "book_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global BOOK_${org_id}_${proj_id}_$id
				puts $F "set BOOK_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset BOOK_${org_id}_${proj_id}_$id name [list [keylget BOOK_${org_id}_${proj_id}_$id name]]\n" 
			}\
			elseif {[regexp "todo_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global TODO_${org_id}_${proj_id}_$id
				puts $F "set TODO_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset TODO_${org_id}_${proj_id}_$id name [list [keylget TODO_${org_id}_${proj_id}_$id name]]\n" 
			}\
			elseif {[regexp "task_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global TASK_${org_id}_${proj_id}_$id
				puts $F "set TASK_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset TASK_${org_id}_${proj_id}_$id name [list [keylget TASK_${org_id}_${proj_id}_$id name]]\n" 
			}\
			elseif {[regexp "budg_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global BUDG_${org_id}_${proj_id}_$id
				puts $F "set BUDG_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset BUDG_${org_id}_${proj_id}_$id name [list [keylget BUDG_${org_id}_${proj_id}_$id name]]\n" 
			}\
			elseif {[regexp "notes_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global NOTES_${org_id}_${proj_id}_$id
				puts $F "set NOTES_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset NOTES_${org_id}_${proj_id}_$id name [list [keylget NOTES_${org_id}_${proj_id}_$id name]]\n" 
			}\
			elseif {[regexp "daily_${org_id}_${proj_id}_(\[0-9\]+)" $f all id]}\
			{
				global DAILY_${org_id}_${proj_id}_$id
				puts $F "set DAILY_${org_id}_${proj_id}_$id \"\""
				puts $F "keylset DAILY_${org_id}_${proj_id}_$id name [list [keylget DAILY_${org_id}_${proj_id}_$id name]]\n" 
			}
		}
		close $F
	} errmsg]
	if {$ct > 0}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc.tmp w+]
		rmdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}
		myError "Cannot save project. Reason:\n$errmsg"
	}\
	else\
	{
		frename $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc.tmp $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc
		chmod 0600 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc
	}
}

proc proj_delete {org_id proj_id}\
{
	global ORG_PATH CUR_PROJ LIST_PROJ
	global PROJ_${org_id}_${proj_id}
	global confirm
	gen_d_yn_confirm "Do you really want to delete \"[keylget PROJ_${org_id}_${proj_id} name]\" and all its dependants ?"
	tkwait variable confirm
	if {$confirm == "yes"}\
	{
		foreach f [readdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}] \
		{
			if {[regexp "book_${org_id}_${proj_id}_(\[0-9\]+)" $f all book_id]}\
			{
				book_delete $org_id $proj_id $book_id
			}
			if {[regexp "todo_${org_id}_${proj_id}_(\[0-9\]+)" $f all todo_id]}\
			{
				todo_delete $org_id $proj_id $todo_id
			}
			if {[regexp "task_${org_id}_${proj_id}_(\[0-9\]+)" $f all task_id]}\
			{
				task_delete $org_id $proj_id $task_id
			}
			if {[regexp "budg_${org_id}_${proj_id}_(\[0-9\]+)" $f all budg_id]}\
			{
				budg_delete $org_id $proj_id $budg_id
			}
			if {[regexp "notes_${org_id}_${proj_id}_(\[0-9\]+)" $f all notes_id]}\
			{
				notes_delete $org_id $proj_id $notes_id
			}
			if {[regexp "daily_${org_id}_${proj_id}_(\[0-9\]+)" $f all daily_id]}\
			{
				daily_delete $org_id $proj_id $daily_id
			}
		}
		if {! [lempty [readdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}]]}\
		{
			myError "The project can't be deleted because it is not empty."
		}\
		else\
		{
			unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}_desc
			rmdir $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}
			set LIST_PROJ [lindex [intersect3 $LIST_PROJ $proj_id] 0]
			unset PROJ_${org_id}_${proj_id}
			o_menu_compose $org_id
			set CUR_TYPE proj
			set CUR_PROJ [lindex $LIST_PROJ 0]
			o_display_proj $org_id [lindex $LIST_PROJ 0]
		}
	}
}

