proc task_init_var {org_id proj_id task_id}\
{
	global TASK_REC_${org_id}_${proj_id}_${task_id}
	set TASK_REC_${org_id}_${proj_id}_${task_id} ""
}

proc task_new_id {org_id proj_id}\
{
	global LIST_TASK
	set task_id 0
	foreach b $LIST_TASK \
	{
		if {[regexp "^${org_id}_${proj_id}_(\[0-9\]+)" $b all id]}\
		{
			if {$id > $task_id}\
			{
				set task_id $id
			}
		}
	}
	return [expr $task_id + 1]
}

proc task_d_new {org_id}\
{
	global proj
	global CUR_ORG LIST_PROJ LIST_TASK
	catch {destroy .newtask}
	toplevel .newtask
	wm title .newtask "New task list"
	wm geometry .newtask +50+50
	frame .newtask.b
	pack .newtask.b -side bottom -pady 5 -padx 5
		button .newtask.b.ok -text Ok -width 10 \
			-command {
				if {[.newtask.l.f3.lb size] == 0}\
				{
					return
				}
				set proj_id [join [lindex [split $proj ---] 0]]
				set task_id [task_new_id $CUR_ORG $proj_id]
				task_init_var $CUR_ORG $proj_id $task_id
				upvar #0 TASK_REC_${CUR_ORG}_${proj_id}_${task_id} TASK_REC
				set TASK_REC ""
				upvar #0 TASK_${CUR_ORG}_${proj_id}_${task_id} TASK
				set TASK ""
				if {[join [.newtask.name.e get]] == ""}\
				{
					keylset TASK name "Untitled"
				}\
				else\
				{
					keylset TASK name [.newtask.name.e get]
				}
				lappend LIST_TASK ${CUR_ORG}_${proj_id}_${task_id}
				task_save $CUR_ORG $proj_id $task_id
				o_menu_compose $CUR_ORG
				destroy .newtask
			}
		pack .newtask.b.ok -side left
		button .newtask.b.cancel -text Cancel -width 10 \
			-command "
				destroy .newtask
			"
		pack .newtask.b.cancel -side left
	frame .newtask.name
	pack .newtask.name -fill x
		label .newtask.name.l -text "Task list's name: "
		pack .newtask.name.l -side left
		entry .newtask.name.e
		pack .newtask.name.e -side left
	frame .newtask.l
	pack .newtask.l -side left -fill y -pady 5 -padx 5
		frame .newtask.l.f3
		pack .newtask.l.f3
			scrollbar .newtask.l.f3.vs -command {.newtask.l.f3.lb yview}
			pack .newtask.l.f3.vs -side right -fill y
			listbox .newtask.l.f3.lb -width 30 -yscrollcommand {.newtask.l.f3.vs set}
			pack .newtask.l.f3.lb -side right -fill y
			foreach p $LIST_PROJ \
			{
				global PROJ_${org_id}_$p
				.newtask.l.f3.lb insert end "$p --- [keylget PROJ_${org_id}_$p name]"
			}
			.newtask.l.f3.lb selection set 0
			set proj [.newtask.l.f3.lb get 0]
			bind .newtask.l.f3.lb <1> {
				update
				set proj [.newtask.l.f3.lb get @%x,%y]
			}
	focus .newtask.name.e
}

proc task_save {org_id proj_id task_id}\
{
	global ORG_PATH
	upvar #0 TASK_${org_id}_${proj_id}_${task_id} TASK
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	set ct [catch {
		set F [open $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}.tmp w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError $errmsg
		return
	}
	set ct [catch {
	puts $F "set TASK_${org_id}_${proj_id}_${task_id} \"\""
	puts $F "\n# STRUCTURE"
	puts $F "keylset TASK_${org_id}_${proj_id}_${task_id} name \"[keylget TASK name]\""
	puts $F "\n# RECORDS"
	puts $F "set TASK_REC_${org_id}_${proj_id}_${task_id} \"\""
	foreach i [keylkeys TASK_REC] \
	{
		puts $F "keylset TASK_REC_${org_id}_${proj_id}_${task_id} $i [list [keylget TASK_REC $i]]"
	}
	close $F
	} errmsg]
	if {$ct > 0}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}.tmp
		myError "Cannot save task. Reason:\n$errmsg"
	}\
	else\
	{
		frename $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}.tmp $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}
		chmod 0600 $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}
	}
}

proc task_export_ascii {org_id proj_id task_id}\
{
	upvar #0 TASK_${org_id}_${proj_id}_${task_id} TASK
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global FILENAME TASK_SORT
	o_wait
	task_save $org_id $proj_id $task_id
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the task list as text. Reason:\n$errmsg"
		return
	}
	puts $F "[keylget TASK name]\n"
	puts $F "+--------+----------------------------------------+----------+----------+"
	set line [format "|%8s|%-40.40s|%10s|%10s|" "Priority" "Description" "Start date" "End date"]
	puts $F "$line"
	puts $F "+--------+----------------------------------------+----------+----------+"
	foreach i [lsort -command "task_sort $org_id $proj_id $task_id $TASK_SORT" [keylkeys TASK_REC]] \
	{
		set description [keylget TASK_REC $i.description]
		if {[clength $description] > 40}\
		{
			set j 40
			while {[cindex $description $j] != " " && $j > 0}\
			{
				incr j -1
			}
			if {$j > 0}\
			{
				set description "[crange $description 0 $j]\n[crange $description $j end]"
			}
		}
		set lnum 1
		foreach desc [split $description "\n"]\
		{
			if {$lnum == 1}\
			{
				set line [format "|%8s|%-40.40s|%10s|%10s|" [replicate "*" [keylget TASK_REC $i.priority]] $desc [keylget TASK_REC $i.start] [keylget TASK_REC $i.end]]
				puts $F "$line"
			}\
			else\
			{
				set line [format "|%8s|%-40.40s|%10s|%10s|" " " $desc " " " "]
				puts $F "$line"
			}
			incr lnum
		}
		puts $F "+--------+----------------------------------------+----------+----------+"
	}
	close $F
	o_idle
}

proc task_export_html {org_id proj_id task_id}\
{
	upvar #0 TASK_${org_id}_${proj_id}_${task_id} TASK
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global FILENAME TASK_COL TASK_SORT
	o_wait
	task_save $org_id $proj_id $task_id
	if {[file extension $FILENAME] != ".html"}\
	{
		append FILENAME ".html"
	}
	set ct [catch {
		set F [open [pwd]/$FILENAME w+]
	} errmsg]
	if {$ct > 0}\
	{
		myError "Cannot save the task list as html. Reason:\n$errmsg"
		return
	}
	puts $F "<html><head>"
	puts $F "<title>[html_text2html [keylget TASK name]]</title></head>"
	puts $F "<body bgcolor=\"#FFFFFF\">"
	puts $F "<h2>[html_text2html [keylget TASK name]]</h2>"
	puts $F "<table>"
	puts $F "\t<tr><td><b>Description</b></td><td><b>Start date</b></td><td><b>End date</b></td></tr>"
	foreach i [lsort -command "task_sort $org_id $proj_id $task_id $TASK_SORT" [keylkeys TASK_REC]] \
	{
		puts $F "\t<tr><td bgcolor=\"$TASK_COL([keylget TASK_REC $i.priority])\">[html_text2html [keylget TASK_REC $i.description]]</td><td bgcolor=\"$TASK_COL([keylget TASK_REC $i.priority])\">[keylget TASK_REC $i.start]</td><td bgcolor=\"$TASK_COL([keylget TASK_REC $i.priority])\">[keylget TASK_REC $i.end]</td></tr>"
	}
	puts $F "</table>"
	puts $F "</body></html>"
	close $F
	o_idle
}

proc task_d_export {org_id proj_id task_id mode}\
{
	global FILENAME fbe 
	catch {destroy .file}
	toplevel .file
	wm title .file "Export task list - $mode"
	wm geometry .file 250x250+100+50
	filebrowser .file.fb 
	pack .file.fb
		if {$mode == "html"}\
		{
			.file.fb addlistbox .*.html -selectmode single
		}\
		else\
		{
			.file.fb addlistbox .* -selectmode single
		}
		.file.fb addlabel -text "Export: "
		.file.fb adddirlabel
		set fbe [.file.fb addentry -width 20]
		.file.fb addbutton -text OK -width 10 \
			-command "
				set FILENAME \[$fbe get\]
				if {\$FILENAME != \"\"}\
				{
					destroy .file
					task_export_$mode $org_id $proj_id $task_id
				}
			"
		.file.fb addbutton -text Cancel -width 10 \
			-command "destroy .file"
		if {$mode == "html"}\
		{
			.file.fb readdir [pwd] .*.html
		}\
		else\
		{
			.file.fb readdir [pwd] .*
		}
	focus $fbe
}

proc task_delete {org_id proj_id task_id}\
{
	global ORG_PATH LIST_TASK CUR_TASK CUR_TYPE
	global TASK_${org_id}_${proj_id}_${task_id}
	global TASK_REC_${org_id}_${proj_id}_${task_id}
	global confirm
	gen_d_yn_confirm "Do you really want to delete \"[keylget TASK_${org_id}_${proj_id}_${task_id} name]\"?"
	tkwait variable confirm
	if {$confirm == "yes"}\
	{
		unlink $ORG_PATH/org_${org_id}/proj_${org_id}_${proj_id}/task_${org_id}_${proj_id}_${task_id}
		set LIST_TASK [lindex [intersect3 $LIST_TASK ${org_id}_${proj_id}_${task_id}] 0]
		unset TASK_${org_id}_${proj_id}_${task_id}
		unset TASK_REC_${org_id}_${proj_id}_${task_id}
		o_menu_compose $org_id
		set CUR_TYPE proj
		o_display_proj $org_id $proj_id
	}
}

##############################

proc task_sort {org_id proj_id task_id field id1 id2}\
{
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	if {$field == "id"}\
	{
		if {$id1 > $id2}\
		{
			return 1
		}\
		elseif {$id1 < $id2}\
		{
			return -1
		}\
		else\
		{
			return 0
		}
	}\
	else\
	{
		return [ccollate [keylget TASK_REC $id1.$field] [keylget TASK_REC $id2.$field]]
	}
}

##############################

proc task_d_ask_description {org_id proj_id task_id id}\
{
	upvar #0 TASK_${org_id}_${proj_id}_${task_id} TASK
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global description priority start end TASK_COL
	catch {destroy .ask}
	toplevel .ask
	wm title .ask "Description"
	wm geometry .ask +100+100
	if {[keylget TASK_REC $id.priority {}]}\
	{
		set priority [keylget TASK_REC $id.priority]
	}\
	else\
	{
		set priority 1
	}
	if {[keylget TASK_REC $id.start {}]}\
	{
		set start [keylget TASK_REC $id.start]
	}\
	else\
	{
		set start [gen_get_cur_year]-[gen_get_cur_month]-[gen_get_cur_day]
	}
	set sy [lindex [split $start -] 0]
	set sm [lindex [split $start -] 1]
	set sd [lindex [split $start -] 2]
	if {[keylget TASK_REC $id.end {}]}\
	{
		set end [keylget TASK_REC $id.end]
	}\
	else\
	{
		set end [gen_get_cur_year]-[gen_get_cur_month]-[gen_get_cur_day]
	}
	set ey [lindex [split $end -] 0]
	set em [lindex [split $end -] 1]
	set ed [lindex [split $end -] 2]
	text .ask.description -width 25 -height 2 -wrap word
	pack .ask.description -side top -fill x -anchor w
	frame .ask.start
	pack .ask.start -side top
		label .ask.start.lb -text "Start date (YYYY MM DD)" -width 24
		pack .ask.start.lb -side left
		entry .ask.start.year -width 4
		pack .ask.start.year -side left
		entry .ask.start.month -width 2
		pack .ask.start.month -side left
		entry .ask.start.day -width 2
		pack .ask.start.day -side left
	.ask.start.year insert 0 $sy
	.ask.start.month insert 0 $sm
	.ask.start.day insert 0 $sd
	frame .ask.end
	pack .ask.end -side top
		label .ask.end.lb -text "End date (YYYY MM DD)" -width 24
		pack .ask.end.lb -side left
		entry .ask.end.year -width 4
		pack .ask.end.year -side left
		entry .ask.end.month -width 2
		pack .ask.end.month -side left
		entry .ask.end.day -width 2
		pack .ask.end.day -side left
	.ask.end.year insert 0 $ey
	.ask.end.month insert 0 $em
	.ask.end.day insert 0 $ed
	frame .ask.priority
	pack .ask.priority -side top
		label .ask.priority.lb -text "Priority "
		pack .ask.priority.lb -side left
		radiobutton .ask.priority.p1 -background $TASK_COL(1) -variable priority -value 1
		pack .ask.priority.p1 -side left
		radiobutton .ask.priority.p2 -background $TASK_COL(2) -variable priority -value 2
		pack .ask.priority.p2 -side left
		radiobutton .ask.priority.p3 -background $TASK_COL(3) -variable priority -value 3
		pack .ask.priority.p3 -side left
		radiobutton .ask.priority.p4 -background $TASK_COL(4) -variable priority -value 4
		pack .ask.priority.p4 -side left
		radiobutton .ask.priority.p5 -background $TASK_COL(5) -variable priority -value 5
		pack .ask.priority.p5 -side left
	frame .ask.b
	pack .ask.b -side bottom
		button .ask.b.ok -text Ok -width 10 \
			-command {
				if {[gen_check_date [.ask.start.year get] [.ask.start.month get] [.ask.start.day get] "Start date"] && [gen_check_date [.ask.end.year get] [.ask.end.month get] [.ask.end.day get] "End date"]}\
				{
					if {[clock scan [.ask.end.month get]/[.ask.end.day get]/[.ask.end.year get]] < [clock scan [.ask.start.month get]/[.ask.start.day get]/[.ask.start.year get]]}\
					{
						myError "End date more recent that start date."
					}\
					else\
					{
						regsub -all "^0*" [.ask.start.year get] "" sy
						regsub -all "^0*" [.ask.start.month get] "" sm
						regsub -all "^0*" [.ask.start.day get] "" sd
						regsub -all "^0*" [.ask.end.year get] "" ey
						regsub -all "^0*" [.ask.end.month get] "" em
						regsub -all "^0*" [.ask.end.day get] "" ed
						set start [format "%04d-%02d-%02d" $sy $sm $sd]
						set end [format "%04d-%02d-%02d" $ey $em $ed]
						set description "[.ask.description get 1.0 end-1c]"
						destroy .ask
					}
				}
			}
		pack .ask.b.ok -side left
		button .ask.b.cancel -text Cancel -width 10
		pack .ask.b.cancel -side left
		button .ask.b.del -text Delete -width 10 \
			-command "
				set description \"\"
				task_del_record $org_id $proj_id $task_id $id
				destroy .ask
			"
		pack .ask.b.del -side left
	if {[keylget TASK_REC $id.description {}]}\
	{
		.ask.description insert 1.0 "[keylget TASK_REC $id.description]"
		.ask.b.cancel configure -command "
			set priority $priority
			set start $start
			set end $end
			set description [list [keylget TASK_REC $id.description]]
			destroy .ask
		"
	}\
	else\
	{
		.ask.b.cancel configure -command "
			set description \"\"
			destroy .ask
		"
	}
	focus .ask.description
}

proc task_popup_description {org_id proj_id task_id id}\
{
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global description priority start end TASK_COL
	task_d_ask_description $org_id $proj_id $task_id $id
	tkwait variable description
	if {$description == ""}\
	{
		if {[keylget TASK_REC $id {}]}\
		{
			keyldel TASK_REC $id
		}
		return
	}
	keylset TASK_REC $id.description "$description"
	keylset TASK_REC $id.priority $priority
	keylset TASK_REC $id.start $start
	keylset TASK_REC $id.end $end
	.r.task.c itemconfigure rect$id -fill $TASK_COL([keylget TASK_REC $id.priority])
	.r.task.c itemconfigure desc$id -text "[keylget TASK_REC $id.description]"
	.r.task.c itemconfigure s$id -text "[keylget TASK_REC $id.start]" -anchor nw
	.r.task.c itemconfigure e$id -text "[keylget TASK_REC $id.end]" -anchor nw
}

proc task_new_record {org_id proj_id task_id}\
{
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global description priority start end CUR_Y TASK_COL
	set CUR_ID [expr [lindex [lsort [keylkeys TASK_REC]] end] + 1]
	task_d_ask_description $org_id $proj_id $task_id $CUR_ID
	tkwait variable description
	if {$description == ""}\
	{
		if {[keylget TASK_REC $CUR_ID {}]}\
		{
			keyldel TASK_REC $CUR_ID
		}
		return
	}
	regsub -all "\n" $description { } description
	keylset TASK_REC $CUR_ID.description "$description"
	keylset TASK_REC $CUR_ID.priority $priority
	keylset TASK_REC $CUR_ID.start $start
	keylset TASK_REC $CUR_ID.end $end
	set tagrect [.r.task.c create rectangle 10 [expr $CUR_Y - 5] 380 [expr $CUR_Y + 35] -fill $TASK_COL([keylget TASK_REC $CUR_ID.priority]) -outline {} -tags rect$CUR_ID]
	set tag1 [.r.task.c create text 20 $CUR_Y -text "[keylget TASK_REC $CUR_ID.description]" -anchor nw -width 200 -tags desc$CUR_ID]
	set tag2 [.r.task.c create text 220 $CUR_Y -text "[keylget TASK_REC $CUR_ID.start]" -anchor nw -fill #339933 -tags s$CUR_ID]
	set tag3 [.r.task.c create text 300 $CUR_Y -text "[keylget TASK_REC $CUR_ID.end]" -fill #993333 -anchor nw -tags e$CUR_ID]
	incr CUR_Y 45
	.r.task.c bind $tagrect <1> "
		task_popup_description $org_id $proj_id $task_id $CUR_ID
	"
	.r.task.c bind $tag1 <1> "
		task_popup_description $org_id $proj_id $task_id $CUR_ID
	"
	.r.task.c bind $tag2 <1> "
		task_popup_description $org_id $proj_id $task_id $CUR_ID
	"
	.r.task.c bind $tag3 <1> "
		task_popup_description $org_id $proj_id $task_id $CUR_ID
	"
	.r.task.c configure -scrollregion [.r.task.c bbox all]
}

proc task_del_record {org_id proj_id task_id id}\
{
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	if {! [keylget TASK_REC $id {}]}\
	{
		return
	}
	keyldel TASK_REC $id
	task_create_list $org_id $proj_id $task_id
}

#############################

proc task_create_list {org_id proj_id task_id}\
{
	upvar #0 TASK_REC_${org_id}_${proj_id}_${task_id} TASK_REC
	global TASK_COL CUR_Y TASK_SORT
	.r.task.c delete all
	.r.task.c create rectangle 0 0 400 40 -fill #EEEEEE -outline {}
	set tag [.r.task.c create text 20 20 -text "Description" -anchor nw]
	.r.task.c bind $tag <1> "
		set TASK_SORT description
		task_create_list $org_id $proj_id $task_id
	"
	set tag [.r.task.c create text 250 20 -text "Start date" -anchor n]
	.r.task.c bind $tag <1> "
		set TASK_SORT start
		task_create_list $org_id $proj_id $task_id
	"
	set tag [.r.task.c create text 330 20 -text "End date" -anchor n]
	.r.task.c bind $tag <1> "
		set TASK_SORT end
		task_create_list $org_id $proj_id $task_id
	"
	set CUR_Y 50
	foreach i [lsort -command "task_sort $org_id $proj_id $task_id $TASK_SORT" [keylkeys TASK_REC]]\
	{
		set tagrect [.r.task.c create rectangle 10 [expr $CUR_Y - 5] 380 [expr $CUR_Y + 35] -fill $TASK_COL([keylget TASK_REC $i.priority]) -outline {} -tags rect$i]
		set tag1 [.r.task.c create text 20 $CUR_Y -text "[keylget TASK_REC $i.description]" -anchor nw -width 200 -tags desc$i]
		set tag2 [.r.task.c create text 220 $CUR_Y -text "[keylget TASK_REC $i.start]" -fill #339933 -anchor nw -tags s$i]
		set tag3 [.r.task.c create text 300 $CUR_Y -text "[keylget TASK_REC $i.end]" -fill #993333 -anchor nw -tags e$i]
		incr CUR_Y 45
		.r.task.c bind $tag1 <1> "
			task_popup_description $org_id $proj_id $task_id $i
		"
		.r.task.c bind $tag2 <1> "
			task_popup_description $org_id $proj_id $task_id $i
		"
		.r.task.c bind $tag3 <1> "
			task_popup_description $org_id $proj_id $task_id $i
		"
		.r.task.c bind $tagrect <1> "
			task_popup_description $org_id $proj_id $task_id $i
		"
	}
	.r.task.c configure -scrollregion [.r.task.c bbox all]
}

